@@ -37,16 +37,16 @@ Typical ChordPro input:
 
     {start_of_chorus}
     Swing [D]low, sweet [G]chari[D]ot,
-    Comin’ for to carry me [A7]home.
+    Comin' for to carry me [A7]home.
     Swing [D7]low, sweet [G]chari[D]ot,
-    Comin’ for to [A7]carry me [D]home.
+    Comin' for to [A7]carry me [D]home.
     {end_of_chorus}
 
     # Verse
     I [D]looked over Jordan, and [G]what did I [D]see,
-    Comin’ for to carry me [A7]home.
-    A [D]band of angels [G]comin’ after [D]me,
-    Comin’ for to [A7]carry me [D]home.
+    Comin' for to carry me [A7]home.
+    A [D]band of angels [G]comin' after [D]me,
+    Comin' for to [A7]carry me [D]home.
 
     {c: Chorus}
 
@@ -1059,6 +1059,11 @@ sub ::runtimeinfo {
 	    push( @p, $cp->display($_) );
 	}
     }
+    for ( qw( XDG_CONFIG_HOME ) ) {
+	if ( defined($ENV{$_}) && $ENV{$_} ne "" ) {
+	    $msg .= sprintf( $fmt, $_, $cp->display($ENV{$_}) );
+	}
+    }
     push( @p,  map { $cp->display($_) } @{ $cp->resdirs } );
     $tag = "Resource path";
     for ( uniq(@p) ) {
@@ -1070,7 +1075,7 @@ sub ::runtimeinfo {
 	   if ( ChordPro::Delegate::ABC::have_xs() ) {
 	       $x = ChordPro::Delegate::ABC::packaged_qjs();
 	       $msg .= sprintf( $fmt, "ABC support",
-				$x->[0] . " (" . $x->[1] . ")" );
+				$x->{desc} . " (" . $x->{version} . ")" );
 	   }
 	   elsif ( $x = findexe( "abc2svg", "silent" )
 		        || findexe( "abcnode", "silent" ) ) {
@@ -1078,7 +1083,7 @@ sub ::runtimeinfo {
 	   }
 	   elsif ( $x = ChordPro::Delegate::ABC::packaged_qjs() ) {
 	       $msg .= sprintf( $fmt, "ABC support",
-				$cp->display($x->[0]) . " (" . $cp->display($x->[-1]) . ")" );
+				$cp->display($x->{desc}) . " (" . $x->{version} . ")" );
 	   }
     };
 
