@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#! perl
 
 package main;
 
@@ -138,19 +138,49 @@ sub generate_songbook {
 	my $l = $ctl->{line};
 	my $start = $start_of{songbook} - $options->{"start-page-number"};
 	my $pgtpl = $ctl->{pageno};
-	my $song =
-	  { title     =>  $t,
-	    meta => { title => [ $t ] },
-	    structure => "linear",
-	    body      => [
-		     map { +{ type    => "tocline",
-			      context => "toc",
-			      title   => fmt_subst( $_->[-1], $l ),
-			      page    => $pr->{pdf}->openpage($_->[-1]->{meta}->{tocpage}+$start),
-			      pageno  => fmt_subst( $_->[-1], $pgtpl ),
-			    } } @$book,
-	    ],
-	  };
+
+	# If we have a template, process it as a song and prepend.
+	my $song;
+	    my $cf;
+	if ( $ctl->{template} ) {
+	    my $tpl = $ctl->{template};
+	    if ( $tpl =~ /\.\w+/ ) { # file
+		$cf = CP->siblingres( $book[0][-1]->{source}->{file},
+				      $tpl, class => "templates" );
+		warn("ToC template not found: $tpl\n") unless $cf;
+	    }
+	    else {
+		$cf = CP->findres( $tpl.".cho", class => "templates" );
+		if ( $verbose ) {
+		    warn("ToC template",
+			 $cf ? " found: $cf" : " not found: $tpl.cho\n")
+		}
+	    }
+	}
+	if ( $cf ) {
+	    my $opts = {};
+	    my $lines = loadlines( $cf, $opts );
+	    $song = ChordPro::Song->new( { %$opts,
+					   generate => 'PDF' } );
+	    my $l = 0;
+	    $song->parse_song( $lines, \$l, {}, {} );
+	    $t = fmt_subst( $book[0][-1], $song->{title} )
+	      if $song->{title};
+	    my $st = fmt_subst( $book[0][-1], $song->{meta}->{subtitle}->[0] )
+	      if $song->{meta}->{subtitle} && $song->{meta}->{subtitle}->[0];
+	}
+	else {
+	    $song = { meta => {} };
+	    $song->{title} //= $t;
+	    $song->{meta}->{title} //= [ $t ];
+	}
+	push( @{ $song->{body} //= [] },
+	      map { +{ type    => "tocline",
+		       context => "toc",
+		       title   => fmt_subst( $_->[-1], $l ),
+		       page    => $pr->{pdf}->openpage($_->[-1]->{meta}->{tocpage}+$start),
+		       pageno  => fmt_subst( $_->[-1], $pgtpl ),
+		     } } @$book );
 
 	# Prepend the toc.
 	$page = generate_song( $song,
@@ -544,11 +574,14 @@ sub generate_song {
     };
 
     my $col;
+    my $spreadimage;
 
     my $col_adjust = sub {
 	if ( $ps->{columns} <= 1 ) {
 	    warn("L=", $ps->{__leftmargin},
 	     ", R=", $ps->{__rightmargin},
+	     ", T=", $ps->{_top},
+	     ", S=", $spreadimage//"<undef>",
 	     "\n") if $config->{debug}->{spacing};
 	    return;
 	}
@@ -559,17 +592,19 @@ sub generate_song {
 	    + $ps->{columnoffsets}->[$col+1];
 	$ps->{__rightmargin} -= $ps->{columnspace}
 	  if $col < $ps->{columns}-1;
+	$y = $ps->{_top};
 	warn("C=$col, L=", $ps->{__leftmargin},
 	     ", R=", $ps->{__rightmargin},
+	     ", T=", $ps->{_top},
+	     ", S=", $spreadimage//"<undef>",
 	     "\n") if $config->{debug}->{spacing};
-	$y = $ps->{_top};
 	$x += $ps->{_indent};
+	$y -= $spreadimage if defined($spreadimage) && !ref($spreadimage);
     };
 
     my $vsp_ignorefirst;
     my $startpage = $opts->{startpage} || 1;
     my $thispage = $startpage - 1;
-    my $spreadimage;
 
     # Physical newpage handler.
     my $newpage = sub {
@@ -651,12 +686,8 @@ sub generate_song {
 	$x = $ps->{__leftmargin};
 	$y = $ps->{_margintop};
 	$y += $ps->{headspace} if $ps->{'head-first-only'} && $class == 2;
-
-	if ( $spreadimage ) {
-	    $y -= imagespread( $spreadimage, $x, $y, $ps );
-	    undef $spreadimage;
-	}
 	$x += $ps->{_indent};
+	undef $spreadimage unless ref($spreadimage);
 	$ps->{_top} = $y;
 	$col = 0;
 	$vsp_ignorefirst = 1;
@@ -859,13 +890,7 @@ sub generate_song {
 
 #    prepare_assets( $s, $pr );
 
-    if ( $s->{spreadimage} ) {
-	$spreadimage = $assets->{$s->{spreadimage}->{id}};
-#	  { type => "image",
-#	    id => $s->{spreadimage}->{id},
-#	    opts => { spread => $s->{spreadimage}->{space} }
-#	  };
-    }
+    $spreadimage = $s->{spreadimage};
 
     # Get going.
     $newpage->();
@@ -905,6 +930,14 @@ sub generate_song {
  	    $chorddiagrams->() unless $dctl->{show} eq "below";
 	    # Prepare the assets now we know the page width.
 	    prepare_assets( $s, $pr );
+
+	    if ( $spreadimage ) {
+		if (ref($spreadimage) eq 'HASH' ) {
+		    # Spread image doesn't indent.
+		    $spreadimage = imagespread( $spreadimage, $x-$ps->{_indent}, $y, $ps );
+		}
+		$y -= $spreadimage;
+	    }
 	    showlayout($ps) if $ps->{showlayout} || $config->{debug}->{spacing};
 	}
 
@@ -1235,9 +1268,7 @@ sub generate_song {
 	    $checkspace->($vsp);
 	    $pr->show_vpos( $y, 0 ) if $config->{debug}->{spacing};
 
-	    tocline( $elt, $x, $y, $ps );
-
-	    $y -= $vsp;
+	    $y -= $vsp * tocline( $elt, $x, $y, $ps );
 	    $pr->show_vpos( $y, 1 ) if $config->{debug}->{spacing};
 	    next;
 	}
@@ -1403,21 +1434,22 @@ sub generate_song {
 
 	# margin* are offsets from the edges of the paper.
 	# _*margin are offsets taking even/odd pages into account.
-	# _margin* are physical coordinates, taking ...
 	if ( $rightpage ) {
 	    $ps->{_leftmargin}  = $ps->{marginleft};
-	    $ps->{_marginleft}  = $ps->{marginleft};
 	    $ps->{_rightmargin} = $ps->{marginright};
-	    $ps->{_marginright} = $ps->{papersize}->[0] - $ps->{marginright};
 	}
 	else {
 	    $ps->{_leftmargin}  = $ps->{marginright};
-	    $ps->{_marginleft}  = $ps->{marginright};
 	    $ps->{_rightmargin} = $ps->{marginleft};
-	    $ps->{_marginright} = $ps->{papersize}->[0] - $ps->{marginleft};
 	}
+
+	# _margin* are physical coordinates, taking even/odd pages into account.
+	$ps->{_marginleft}    = $ps->{_leftmargin};
+	$ps->{_marginright}   = $ps->{papersize}->[0] - $ps->{_rightmargin};
 	$ps->{_marginbottom}  = $ps->{marginbottom};
 	$ps->{_margintop}     = $ps->{papersize}->[1] - $ps->{margintop};
+
+	# Bottom margin, taking bottom chords into account.
 	$ps->{_bottommargin}  = $ps->{marginbottom};
 
 	# Physical coordinates; will be adjusted to columns if needed.
@@ -1869,7 +1901,6 @@ sub imageline {
     else {
 	$pw = $ps->{__rightmargin} - $ps->{_leftmargin};
     }
-    $pw -= $ps->{_indent};
     my $ph = $ps->{_margintop} - $ps->{_marginbottom};
 
     if ( $width && $width =~ /^(\d+(?:\.\d+)?)\%$/ ) {
@@ -1999,8 +2030,7 @@ sub imageline {
 		  $ox//0, $oy//0, $align,
 		 )) if $config->{debug}->{images};
 
-#	$pr->add_image( $img, $x, $y, $w, $h, $opts->{border} || 0 );
-    $pr->add_object( $img, $x-2, $y,
+    $pr->add_object( $img, $x, $y,
 		     xscale => $w/$img->width,
 		     yscale => $h/$img->height,
 		     border => $opts->{border} || 0,
@@ -2015,20 +2045,19 @@ sub imageline {
 }
 
 sub imagespread {
-    my ( $elt, $x, $y, $ps ) = @_;
-    ::dump($elt);
-    my $opts = $elt->{opts};
+    my ( $si, $x, $y, $ps ) = @_;
     my $pr = $ps->{pr};
 
-    my $tag = "id=" . $elt->{opts}->{id};
+    my $tag = "id=" . $si->{id};
     return "Unknown asset: $tag"
-      unless exists( $assets->{$elt->{opts}->{id}} );
-    my $img = $assets->{$elt->{opts}->{id}}->{data};
+      unless exists( $assets->{$si->{id}} );
+    my $img = $assets->{$si->{id}}->{data};
     return "Unhandled asset: $tag"
       unless $img;
+    my $opts = {};
 
     # Available width and height.
-    my $pw = $ps->{__rightmargin} - $ps->{_leftmargin};
+    my $pw = $ps->{_marginright} - $ps->{_marginleft};
     my $ph = $ps->{_margintop} - $ps->{_marginbottom};
 
     my $scale = 1;
@@ -2090,7 +2119,7 @@ sub imagespread {
 		     align  => $align,
 		   );
 
-    return $h + $elt->{opts}->{spread};			# vertical size
+    return $h + $si->{space};			# vertical size
 }
 
 sub tocline {
@@ -2104,19 +2133,36 @@ sub tocline {
     $pr->setfont($ftoc);
     my $tpl = $elt->{title};
     my $vsp;
-    for ( split( /\\n/, $tpl ) ) {
-	$ps->{pr}->text( $_, $x, $y );
+    my $lines = 0;
+
+    my $p = $elt->{pageno};
+    my $pw = $pr->strwidth($p);
+    my $ww = $ps->{__rightmargin} - $x - $pr->strwidth("xxx$p");
+    for my $text ( split( /\\n/, $tpl ) ) {
+	$lines++;
+	# Suppress unclosed markup warnings.
+	local $SIG{__WARN__} = sub{
+	    CORE::warn(@_) unless "@_" =~ /Unclosed markup/;
+	};
+	# Get the part that fits (hopefully, all) and print.
+	( $text, my $ex ) = @{ defrag( [ $pr->wrap( $text, $ww ) ] ) };
+	$pr->text( $text, $x, $y );
 	unless ($vsp) {
-	    my $p = $elt->{pageno};
-	    $ps->{pr}->text( $p, $ps->{__rightmargin} - $pr->strwidth($p), $y );
+	    $ps->{pr}->text( $p, $ps->{__rightmargin} - $pw, $y );
 	    $vsp = _vsp("toc", $ps);
+	    $x += $pr->strwidth( $config->{settings}->{wrapindent} )
+	      if $ex ne "";
 	}
 	$y -= $vsp;
+	if ( $ex ne "" ) {
+	    $text = $ex;
+	    redo;
+	}
     }
     my $ann = $pr->{pdfpage}->annotation;
     $ann->link($elt->{page});
-    $ann->rect( $ps->{__leftmargin}, $y0 - $ftoc->{size} * $ps->{spacing}->{toc},
-		$ps->{__rightmargin}, $y0 );
+    $ann->rect( $ps->{__leftmargin}, $y0-$lines*$vsp, $ps->{__rightmargin}, $y0 );
+    return $lines;
 }
 
 sub has_visible_chords {
@@ -2235,7 +2281,7 @@ sub set_columns {
 
     if ( @cols ) {		# columns with explicit widths
 	my $stars;
-	my $wx = $w;		# available
+	my $wx = $w + $ps->{columnspace}; # available
 	for ( @cols ) {
 	    if ( !$_ || $_ eq '*' ) {
 		$stars++;
@@ -2283,6 +2329,12 @@ sub showlayout {
     my $mr = $ps->{_rightmargin};
     my $ml = $ps->{_leftmargin};
 
+    my $f = sub {
+	my $t = sprintf( "%.1f", shift );
+	$t =~ s/\.0$//;
+	return $t;
+    };
+
     $pr->rectxy( $ml,
 		 $ps->{marginbottom},
 		 $ps->{papersize}->[0]-$mr,
@@ -2294,16 +2346,16 @@ sub showlayout {
     $pr->setfont($font,$fsz);
     $pr->text( "<span color='red'>$ml</span>",
 	       $ml, $ptop, $font, $fsz );
-    my $t = $ps->{papersize}->[0]-$mr;
+    my $t = $f->($ps->{papersize}->[0]-$mr);
     $pr->text( "<span color='red'>$t</span>",
 	       $ps->{papersize}->[0]-$mr-$pr->strwidth("$mr"),
 	       $ptop, $font, $fsz );
-    $t = $ps->{papersize}->[1]-$ps->{margintop};
+    $t = $f->($ps->{papersize}->[1]-$ps->{margintop});
     $pr->text( "<span color='red'>$t  </span>",
 	       $ml-$pr->strwidth("$t  "),
 	       $ps->{papersize}->[1]-$ps->{margintop}-2,
 	       $font, $fsz );
-    $t = $ps->{marginbottom};
+    $t = $f->($ps->{marginbottom});
     $pr->text( "<span color='red'>$t  </span>",
 	       $ml-$pr->strwidth("$t  "),
 	       $ps->{marginbottom}-2,
@@ -2313,14 +2365,14 @@ sub showlayout {
 	      $ps->{papersize}->[0]-$ml-$mr,
 	      $lw, $col );
     $pr->hline(@a);
-    $t = $a[1];
+    $t = $f->($a[1]);
     $pr->text( "<span color='red'>$t  </span>",
 	       $ml-$pr->strwidth("$t  "),
 	       $a[1]-2,
 	       $font, $fsz );
     $a[1] = $ps->{marginbottom}-$ps->{footspace};
     $pr->hline(@a);
-    $t = $a[1];
+    $t = $f->($a[1]);
     $pr->text( "<span color='red'>$t  </span>",
 	       $ml-$pr->strwidth("$t  "),
 	       $a[1]-2,
@@ -2335,11 +2387,11 @@ sub showlayout {
 	   $lw, $col );
     foreach my $i ( 0 .. @off-1 ) {
 	next unless $off[$i];
-	$a[0] = $ml + $off[$i];
+	$a[0] = $f->($ml + $off[$i]);
 	$pr->text( "<span color='red'>$a[0]</span>",
 		   $a[0] - $pr->strwidth($a[0])/2, $ptop, $font, $fsz );
 	$pr->vline(@a);
-	$a[0] = $ml + $off[$i] - $ps->{columnspace};
+	$a[0] = $f->($ml + $off[$i] - $ps->{columnspace});
 	$pr->text( "<span color='red'>$a[0]</span>",
 		   $a[0] - $pr->strwidth($a[0])/2, $ptop, $font, $fsz );
 	$pr->vline(@a);
@@ -2645,15 +2697,16 @@ sub prepare_assets {
     my ( $s, $pr ) = @_;
 
     my %sa = %{$s->{assets}//{}};	# song assets
+    $s->{_ps} = $pr->{ps};		# for handlers TODO
 
     # All elements generate zero or one display items, except for SVG images
     # than can result in a series of display items.
     # So we first scan the list for SVG and delegate items and turn these
     # into simple display items.
 
-    my $pw = $ps->{__rightmargin} - $ps->{__leftmargin}
+    my $pw = $ps->{_marginright} - $ps->{_marginleft};
+    my $cw = ( $pw - ( $ps->{columns} - 1 ) * $ps->{columnspace} ) /$ps->{columns}
       - $ps->{_indent};
-    my $cw = ( $pw - ( $ps->{columns} - 1 ) * $ps->{columnspace} ) /$ps->{columns};
     warn("PDF: Preparing ", scalar(keys %sa), " image",
 	 keys(%sa) == 1 ? "" : "s", ", pw=$pw, cw=$cw\n")
       if $config->{debug}->{images} || $config->{debug}->{assets};
@@ -2665,7 +2718,9 @@ sub prepare_assets {
 	if ( $elt->{type} eq "image" && $elt->{subtype} eq "delegate" ) {
 	    my $delegate = $elt->{delegate};
 	    warn("PDF: Preparing delegate $delegate, handler ",
-		 $elt->{handler}, "\n") if $config->{debug}->{images};
+		 $elt->{handler},
+		 ( map { " $_=" . $elt->{opts}->{$_} } keys(%{$elt->{opts}//{}})),
+		 "\n") if $config->{debug}->{images};
 
 	    my $pkg = __PACKAGE__;
 	    $pkg =~ s/::Output::[:\w]+$/::Delegate::$delegate/;
@@ -2678,18 +2733,20 @@ sub prepare_assets {
 
 	    # Determine actual width.
 	    my $w = defined($elt->{opts}->{spread}) ? $pw : $cw;
-#	    $w -= $ps->{_indent};
 	    $w = $elt->{opts}->{width}
 	      if $elt->{opts}->{width} && $elt->{opts}->{width} < $w;
 
 	    my $res = $hd->( $s, $w, $elt );
 	    if ( $res ) {
+		$res->{opts} = { %{ $res->{opts} // {} },
+				 %{ $elt->{opts} // {} } };
 		warn( "PDF: Preparing delegate $delegate, handler ",
 		      $elt->{handler}, " => ",
-		      $res->{type}, "/", $res->{subtype}, "\n" )
+		      $res->{type}, "/", $res->{subtype},
+		      ( map { " $_=" . $res->{opts}->{$_} } keys(%{$res->{opts}//{}})),
+		      " w=$w",
+		      "\n" )
 		  if $config->{debug}->{images};
-		$res->{opts} = { %{ $res->{opts}   // {} },
-				 %{ $elt->{opts} // {} } };
 		$s->{assets}->{$id} = $res;
 	    }
 
@@ -2828,7 +2885,8 @@ sub prepare_assets {
 					};
 		warn("Created asset $id ($elt->{subtype}, ",
 		     $res->{width}, "x", $res->{height}, ")",
-		     map { " $_=" . $res->{opts}->{$_} } keys( %{$res->{opts}//{}} ),
+		     ( map { " $_=" . $res->{opts}->{$_} }
+		           keys( %{$res->{opts}//{}} ) ),
 		     "\n")
 		  if $config->{debug}->{images};
 	    }
@@ -2898,7 +2956,8 @@ sub is_corefont {
 
 # Font handler for SVG embedding.
 sub svg_fonthandler {
-    my ( $ps, $el, $pdf, $style ) = @_;
+    my ( $ps, $svg, %args ) = @_;
+    my ( $pdf, $style ) = @args{qw(pdf style)};
 
     my $family = lc( $style->{'font-family'} );
     my $stl    = lc( $style->{'font-style'}  // "normal" );
@@ -2958,15 +3017,21 @@ sub svg_fonthandler {
 
 # Text handler for SVG embedding.
 sub svg_texthandler {
-    my ( $ps, $el, $xo, $pdf, $style, $text, %opts ) = @_;
+    my ( $ps, $svg, %args ) = @_;
+    my $xo    = delete($args{xo});
+    my $pdf   = delete($args{pdf});
+    my $style = delete($args{style});
+    my $text  = delete($args{text});
+    my %opts  = %args;
+
     my @t = split( /([♯♭])/, $text );
     if ( @t == 1 ) {
 	# Nothing special.
-	$el->set_font( $xo, $style );
+	$svg->set_font( $xo, $style );
 	return $xo->text( $text, %opts );
     }
 
-    my ( $font, $sz ) = $el->root->fontmanager->find_font($style);
+    my ( $font, $sz ) = $svg->root->fontmanager->find_font($style);
     my $has_sharp = $font->glyphByUni(ord("♯")) ne ".notdef";
     my $has_flat  = $font->glyphByUni(ord("♭")) ne ".notdef";
     # For convenience we assume that either both are available, or missing.
